--------------------------------------------------------------------------------
--
--
--	   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--	   SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--	   XILINX DEVICES.	BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--	   AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--	   OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--	   IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--	   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--	   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--	   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--	   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--	   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--	   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--	   FOR A PARTICULAR PURPOSE.
--	   
--	   (c) Copyright 2005 Xilinx, Inc.
--	   All rights reserved.
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Filename:	 ocm_temac_sz410.vhd
-- 
-- Description:  ocm temac for suzaku-v4
--
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Author:		matsu
-- History: 	2005.8.10 - Initial release
--				2008.1.1  - ocm_temac for SUZAKU-V4 release
--				2008.3.27 - bug fix : RX_FIFO reset 
--				2009.4.16 - bug fix : Change behavior of RX_FIFO WR_EN at RX_FIFO state of Full
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

entity ocm_temac_sz410 is
	generic (
		C_BASE_NUM_BITS  : integer range 0 to 9 := 7;
		C_DCR_AWIDTH	 : integer := 10;
		C_DCR_DWIDTH	 : integer := 32;
		C_BASEADDR		 : std_logic_vector(0 to 9) := "0000000000";
		C_HIGHADDR		 : std_logic_vector(0 to 9) := "0000000111";
		C_TX_FIFO_DEPTH  : std_logic_vector(0 to 31) := x"00002000";
		C_RX_FIFO_DEPTH  : std_logic_vector(0 to 31) := x"00002000"
	);
	port (
		mii_rx_clk		 : in  std_logic;
		mii_rx_dv		 : in  std_logic;
		mii_rx_er		 : in  std_logic;
		mii_rxd 		 : in  std_logic_vector (3 downto 0);
		mii_tx_clk		 : in  std_logic ;
		mii_tx_en		 : out std_logic;
		mii_txd 		 : out std_logic_vector(3 downto 0);
		phy_mii_int_n	 : in  std_logic;
		DSOCMBRAMWRDBUS  : in  std_logic_vector (0 to 31);
		ISOCMBRAMWRDBUS  : in  std_logic_vector (0 to 31);
		BRAMDSOCMRDDBUS  : out std_logic_vector (0 to 31);
		MDC_0			 : out std_logic;
		MDIO_OUT_0		 : out std_logic;
		MDIO_IN_0		 : in  std_logic;
		MDIO_TRI_0		 : out std_logic;
		sys_rst 		 : in  std_logic;
		phy_mii_int 	 : out std_logic;
		host_clk		 : in  std_logic;
		rx_fifo_rden	 : in  std_logic;
		tx_fifo_wren	 : in  std_logic;
		tx_fifo_full	 : out std_logic;
		phy_rstn		 : out std_logic;
		fifo_int		 : out std_logic;
		DCR_Read		 : in  std_logic;
		DCR_Write		 : in  std_logic;
		DCR_ABus		 : in  std_logic_vector(0 to C_DCR_AWIDTH - 1);
		DCR_DBus		 : in  std_logic_vector(0 to C_DCR_DWIDTH - 1);
		temac_dcrDBus	 : out std_logic_vector(0 to C_DCR_DWIDTH - 1);
		temac_dcrAck	 : out std_logic;
		DSOCMBRAMABUS	 : in  std_logic_vector(8 to 29)
	);
end ocm_temac_sz410;

architecture STRUCTURE of ocm_temac_sz410 is

attribute box_type : STRING;

component fifo_9_tx
	port (
		rst: in std_logic;
		wr_clk: in std_logic;
		wr_en: in std_logic;
		din: in std_logic_vector(17 downto 0);
		full: out std_logic;
		overflow: out std_logic;
		wr_data_count: out std_logic_vector(11 downto 0);
		rd_clk: in std_logic;
		rd_en: in std_logic;
		dout: out std_logic_vector(8 downto 0);
		empty: out std_logic
	);
end component;

component fifo_9_rx
	port (
		rst: in std_logic;
		wr_clk: in std_logic;
		wr_en: in std_logic;
		din: in std_logic_vector(8 downto 0);
		full: out std_logic;
		rd_clk: in std_logic;
		rd_en: in std_logic;
		dout: out std_logic_vector(17 downto 0);
		empty: out std_logic;
		underflow: out std_logic;
		rd_data_count: out std_logic_vector(11 downto 0)
	);
end component;

component fifo_1_done
	port (
		rst: in std_logic;
		wr_clk: in std_logic;
		wr_en: in std_logic;
		din: in std_logic_vector(0 downto 0);
		rd_clk: in std_logic;
		rd_en: in std_logic;
		dout: out std_logic_vector(0 downto 0);
		empty: out std_logic;
		full: out std_logic
	);
end component;

component temac is
	port(
		-- Client Receiver Interface - EMAC0
		EMAC0CLIENTRXCLIENTCLKOUT		: out std_logic;
		CLIENTEMAC0RXCLIENTCLKIN		: in  std_logic;
		EMAC0CLIENTRXD					: out std_logic_vector(7 downto 0);
		EMAC0CLIENTRXDVLD				: out std_logic;
		EMAC0CLIENTRXDVLDMSW			: out std_logic;
		EMAC0CLIENTRXGOODFRAME			: out std_logic;
		EMAC0CLIENTRXBADFRAME			: out std_logic;
		EMAC0CLIENTRXFRAMEDROP			: out std_logic;
		EMAC0CLIENTRXDVREG6 			: out std_logic;
		EMAC0CLIENTRXSTATS				: out std_logic_vector(6 downto 0);
		EMAC0CLIENTRXSTATSVLD			: out std_logic;
		EMAC0CLIENTRXSTATSBYTEVLD		: out std_logic;

		-- Client Transmitter Interface - EMAC0
		EMAC0CLIENTTXCLIENTCLKOUT		: out std_logic;
		CLIENTEMAC0TXCLIENTCLKIN		: in  std_logic;
		CLIENTEMAC0TXD					: in  std_logic_vector(7 downto 0);
		CLIENTEMAC0TXDVLD				: in  std_logic;
		CLIENTEMAC0TXDVLDMSW			: in  std_logic;
		EMAC0CLIENTTXACK				: out std_logic;
		CLIENTEMAC0TXFIRSTBYTE			: in  std_logic;
		CLIENTEMAC0TXUNDERRUN			: in  std_logic;
		EMAC0CLIENTTXCOLLISION			: out std_logic;
		EMAC0CLIENTTXRETRANSMIT 		: out std_logic;
		CLIENTEMAC0TXIFGDELAY			: in  std_logic_vector(7 downto 0);
		EMAC0CLIENTTXSTATS				: out std_logic;
		EMAC0CLIENTTXSTATSVLD			: out std_logic;
		EMAC0CLIENTTXSTATSBYTEVLD		: out std_logic;

		-- MAC Control Interface - EMAC0
		CLIENTEMAC0PAUSEREQ 			: in  std_logic;
		CLIENTEMAC0PAUSEVAL 			: in  std_logic_vector(15 downto 0);

		-- Clock Signal - EMAC0
		GTX_CLK_0						: in  std_logic;
		EMAC0CLIENTTXGMIIMIICLKOUT		: out std_logic;
		CLIENTEMAC0TXGMIIMIICLKIN		: in  std_logic ;

		-- GMII Interface - EMAC0
		GMII_TXD_0						: out std_logic_vector(7 downto 0);
		GMII_TX_EN_0					: out std_logic;
		GMII_TX_ER_0					: out std_logic;
		GMII_TX_CLK_0					: out std_logic;
		GMII_RXD_0						: in  std_logic_vector(7 downto 0);
		GMII_RX_DV_0					: in  std_logic;
		GMII_RX_ER_0					: in  std_logic;
		GMII_RX_CLK_0					: in  std_logic;

		MII_TX_CLK_0					: in  std_logic;
		GMII_COL_0						: in  std_logic;
		GMII_CRS_0						: in  std_logic;

		-- MDIO Interface - EMAC0
		MDC_0							: out std_logic;
		MDIO_IN_0						: in  std_logic;
		MDIO_OUT_0						: out std_logic;
		MDIO_TRI_0						: out std_logic;

		HOSTCLK 						: in  std_logic;
		-- Asynchronous Reset
		RESET							: in  std_logic
		);
end component;



-- EMAC signal
	signal gnd_i						: std_logic;
	signal reset_i						: std_logic;
	signal rx_client_clk_out_0_i		: std_logic;
	signal rx_client_clk_in_0_i 		: std_logic;
	signal tx_client_clk_out_0_i		: std_logic;
	signal tx_client_clk_in_0_i 		: std_logic;
	signal mii_tx_en_0_i				: std_logic;
	signal mii_txd_0_i					: std_logic_vector(7 downto 0);
	signal mii_tx_en_0_r				: std_logic;
	signal mii_txd_0_r					: std_logic_vector(3 downto 0);
	signal mii_tx_clk_0_i				: std_logic;
	signal mii_rx_clk_ibufg_0_i 		: std_logic;
	signal mii_rx_clk_0_i				: std_logic;
	signal mii_rx_dv_0_i				: std_logic;
	signal mii_rx_er_0_i				: std_logic;
	signal mii_rxd_0_i					: std_logic_vector(3 downto 0);
	signal mii_rx_dv_0_r				: std_logic;
	signal mii_rx_er_0_r				: std_logic;
	signal mii_rxd_0_r					: std_logic_vector(7 downto 0);

-- MDIO signals
	signal mdc_0_obuf					: std_logic;
	signal emac_mdc 					: std_logic;
	signal emac_mdio_in 				: std_logic;
	signal emac_mdio_out				: std_logic;
	signal emac_mdio_tri				: std_logic;
-- phy interrupt
	signal phy_mii_int_n_IBUF			: std_logic_vector (0 to 0);

-- EMAC Receive interface to rx_fifo
	signal rx_valid 			: std_logic;
	signal rx_good_frame		: std_logic;
	signal rx_bad_frame			: std_logic;
	signal rx_data				: std_logic_vector (0 to 7);
	signal rx_clk				: std_logic;
	signal rx_fifo_DO			: std_logic_vector (7 downto 0);
	signal rx_fifo_wren			: std_logic;
	signal rx_fifo_full			: std_logic;
	signal rx_valid_fifo		: std_logic;
	signal rx_fifo_rderr		: std_logic;

-- EMAC Transmit interface to rx_fifo
	signal tx_data				: std_logic_vector (7 downto 0);
	signal tx_valid 			: std_logic;
	signal n_tx_valid			: std_logic;
	signal tx_ack				: std_logic;
	signal tx_ack_delay 		: std_logic;
	signal tx_fifo_empty		: std_logic;
	signal tx_fifo_rden 		: std_logic;
	signal tx_fifo_di			: std_logic_vector (7 downto 0);
	signal tx_fifo_full_wire	: std_logic;
	signal tx_clk				: std_logic;
	signal tx_fifo_rden_start	: std_logic;
	signal n_tx_fifo_di8		: std_logic;

	signal rd_reg				: std_logic;
	signal wr_reg				: std_logic;
	signal rd_edge				: std_logic;
	signal wr_edge				: std_logic;
	signal select_reg			: std_logic;
	signal valid_address		: std_logic;
	signal ack					: std_logic;
	signal rd_qual				: std_logic;
	signal wr_qual				: std_logic;
	signal mux_ctl				: std_logic;
	signal data_out 			: std_logic_vector(0 to C_DCR_DWIDTH - 1);
	signal intreg				: std_logic_vector(0 to C_DCR_DWIDTH - 1);
	signal ioreg				: std_logic_vector(0 to C_DCR_DWIDTH - 1);

	signal tx_full_reg			: std_logic;
	signal tx_empty_reg			: std_logic;
	signal tx_overrun_reg		: std_logic;
	signal rx_full_reg			: std_logic;
	signal rx_empty_reg			: std_logic;
	signal rx_underrun_reg		: std_logic;
	signal tx_valid_reg			: std_logic;
	signal rx_valid_reg			: std_logic;

	signal rx_fifo_reset		: std_logic;
	signal rx_fifo_reset_reg	: std_logic;
	signal rx_fifo_reset_latch	: std_logic;

	signal tx_fifo_overflow			: std_logic;
	signal tx_fifo_wr_data_count	: std_logic_vector(11 downto 0);
	signal rx_fifo_rd_data_count	: std_logic_vector(11 downto 0);
	signal rx_fifo_empty			: std_logic;

	signal rx_done_fifo_wren		: std_logic;
	signal rx_done_fifo_DO			: std_logic;
	signal rx_done_fifo_empty		: std_logic;
	signal rx_done_fifo_rden		: std_logic;
	signal rx_done_fifo_empty_reg	: std_logic;
	signal rx_fifo_rden_one 		: std_logic;
	signal rx_fifo_DO_2 			: std_logic_vector(7 downto 0);
	signal rx_valid_fifo_2			: std_logic;
	signal rx_fifo_parity			: std_logic;
	signal rx_frame_end_reg 		: std_logic;
	signal tx_fifo_di_2 			: std_logic_vector(7 downto 0);
	signal n_tx_fifo_di8_2			: std_logic;

	attribute IOBDELAY_TYPE  : string;
	attribute IOBDELAY_VALUE : integer;
	
begin

	process(host_clk)
	begin
		if host_clk'event and host_clk = '1'  then
			rd_reg <= DCR_Read;
			wr_reg <= DCR_Write;
			ack <= valid_address and select_reg;
			tx_full_reg <= tx_fifo_full_wire;
			tx_empty_reg <= tx_fifo_empty;
			tx_overrun_reg <= tx_fifo_overflow;
			rx_full_reg <= rx_fifo_full;
			rx_empty_reg <= rx_fifo_empty;
			rx_underrun_reg <= rx_fifo_rderr;
			tx_valid_reg <= tx_valid;
			rx_done_fifo_empty_reg <= rx_done_fifo_empty;
		end if;
	end process;

	process (host_clk)
	begin
		if host_clk'event and host_clk = '1' then
			if sys_rst = '1' then
				rx_fifo_reset_reg <= '0';
			else
				rx_fifo_reset_reg <= ISOCMBRAMWRDBUS(0);
			end if;
		end if;
	end process;

	process (host_clk)
	begin
		if host_clk'event and host_clk = '1' then
			if sys_rst = '1' then
				rx_fifo_reset_latch <= '0';
			else
				if ISOCMBRAMWRDBUS(0) = '1' and rx_fifo_reset_reg = '0' then
					rx_fifo_reset_latch <= '1';
				elsif rx_fifo_reset = '1' then
					rx_fifo_reset_latch <= '0';
				end if;
			end if;
		end if;
	end process;

	process (rx_clk)
	begin
		if rx_clk'event and rx_clk = '1' then
			if sys_rst = '1' then
				rx_fifo_reset <= '0';
			else
				if rx_fifo_reset_latch = '1' then
					rx_fifo_reset <= '1';
				else
					rx_fifo_reset <= '0';
				end if;
			end if;
		end if;
	end process;

-- DCR register logic

	-- tx_full register set/clear
	process (host_clk,tx_full_reg,tx_fifo_full_wire)
	begin
		if tx_full_reg = '0' and tx_fifo_full_wire = '1' then
			ioreg(20) <= '1';
		elsif host_clk'event and host_clk = '1' then
			if DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "000" and wr_edge = '1' and DCR_DBus(20) = '1' then
				ioreg(20) <= '0';
			end if;
		end if;
	end process;
	-- tx_empty register set/clear
	process (host_clk,tx_empty_reg,tx_fifo_empty)
	begin
		if tx_empty_reg = '0' and tx_fifo_empty = '1' then
			ioreg(21) <= '1';
		elsif host_clk'event and host_clk = '1' then
			if DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "000" and wr_edge = '1' and DCR_DBus(21) = '1' then
				ioreg(21) <= '0';
			end if;
		end if;
	end process;
	-- tx_overrun register set/clear
	process (host_clk,tx_overrun_reg,tx_fifo_overflow)
	begin
		if tx_overrun_reg = '0' and tx_fifo_overflow = '1' then
			ioreg(22) <= '1';
		elsif host_clk'event and host_clk = '1' then
			if DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "000" and wr_edge = '1' and DCR_DBus(22) = '1' then
				ioreg(22) <= '0';
			end if;
		end if;
	end process;
	-- rx_full register set/clear
	process (host_clk,rx_full_reg,rx_fifo_full)
	begin
		if rx_full_reg = '0' and rx_fifo_full = '1' then
			ioreg(24) <= '1';
		elsif host_clk'event and host_clk = '1' then
			if DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "000" and wr_edge = '1' and DCR_DBus(24) = '1' then
				ioreg(24) <= '0';
			end if;
		end if;
	end process;
	-- rx_empty register set/clear
	process (host_clk,rx_empty_reg,rx_fifo_empty)
	begin
		if rx_empty_reg = '0' and rx_fifo_empty = '1' then
			ioreg(25) <= '1';
		elsif host_clk'event and host_clk = '1' then
			if DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "000" and wr_edge = '1' and DCR_DBus(25) = '1' then
				ioreg(25) <= '0';
			end if;
		end if;
	end process;
	-- rx_underrun register set/clear
	process (host_clk,rx_underrun_reg,rx_fifo_rderr)
	begin
		if rx_underrun_reg = '0' and rx_fifo_rderr = '1' then
			ioreg(26) <= '1';
		elsif host_clk'event and host_clk = '1' then
			if DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "000" and wr_edge = '1' and DCR_DBus(26) = '1' then
				ioreg(26) <= '0';
			end if;
		end if;
	end process;
	-- tx_done register set/clear
	process (host_clk,tx_valid_reg,tx_valid)
	begin
		if tx_valid_reg = '1' and tx_valid = '0' then
			ioreg(30) <= '1';
		elsif host_clk'event and host_clk = '1' then
			if DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "000" and wr_edge = '1' and DCR_DBus(30) = '1' then
				ioreg(30) <= '0';
			end if;
		end if;
	end process;
	-- rx_done register set/clear
	process (host_clk)
	begin
		if host_clk'event and host_clk = '1' then
			if DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "000" and wr_edge = '1' and DCR_DBus(31) = '1' then
				rx_done_fifo_rden <= '1';
			else
				rx_done_fifo_rden <= '0';
				
			end if;
		end if;
	end process;
	-- int enable register
	process (host_clk,sys_rst)
	begin
		if sys_rst = '1' then
			intreg <= (others=>'0');
		elsif host_clk'event and host_clk = '1' then
			if DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "001" and wr_edge = '1' then
				intreg <= DCR_DBus;
			end if;
		end if;
	end process;

	select_reg <= rd_reg or wr_reg;
	valid_address <= '1' when DCR_ABus(0 to C_BASE_NUM_BITS-1) = 
							  C_BASEADDR(0 to C_BASE_NUM_BITS-1) else
					 '0';

	rd_qual <= rd_reg and valid_address;
	wr_qual <= wr_reg and valid_address;

	rd_edge <= (not rd_reg) and DCR_Read and valid_address;
	wr_edge <= (not wr_reg) and DCR_Write and valid_address;

	mux_ctl <= ack and rd_qual;

	temac_dcrDBus <= data_out when mux_ctl = '1' else DCR_DBus;
	temac_dcrAck <= ack;

	data_out <= ioreg when DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "000" else
				intreg when DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "001" else
				x"0000" & "000" & tx_fifo_wr_data_count & '0' when DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "010" else
				x"0000" & "000" & rx_fifo_rd_data_count & '0' when DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "011" else
				C_TX_FIFO_DEPTH when DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "100" else
				C_RX_FIFO_DEPTH when DCR_ABus(C_BASE_NUM_BITS to C_DCR_AWIDTH - 1) = "101" else
				"0" & rx_done_fifo_empty & rx_done_fifo_DO & "0" & x"EADBEEF";

-- interrupt logic

	fifo_int <= (ioreg(30) and intreg(30)) or (ioreg(31) and intreg(31));

-- rx fifo supporting logic

	rx_done_fifo_wren <= rx_valid_reg and (not rx_valid);
	ioreg(31) <= not rx_done_fifo_empty_reg;
	rx_fifo_rden_one <= rx_fifo_rden and (not DSOCMBRAMABUS(29));

	-- last unvalid data write signals
	process (rx_clk)
	begin
		if rx_clk'event and rx_clk = '1'  then
			if sys_rst = '1' then
				rx_valid_reg <= '0';
				rx_frame_end_reg <= '0';
			else
				rx_valid_reg <= rx_valid;
				rx_frame_end_reg <= rx_good_frame or rx_bad_frame;
			end if;
		end if;
	end process;

	-- rx_fifo 2byte align check signal
	process (rx_clk, sys_rst, rx_fifo_reset)
	begin
		if sys_rst = '1' or rx_fifo_reset = '1' then
			rx_fifo_parity <= '0';
		elsif rx_clk'event and rx_clk = '1' then
			if rx_fifo_wren = '1' then
				rx_fifo_parity <= not rx_fifo_parity;
			else
				rx_fifo_parity <= rx_fifo_parity;
			end if;
		end if;
	end process;

-- tx fifo supporting logic

	tx_fifo_full <= tx_fifo_full_wire;
	tx_fifo_di	 <= DSOCMBRAMWRDBUS(24 to 31);
	tx_fifo_di_2 <= DSOCMBRAMWRDBUS(16 to 23);
	tx_fifo_rden <= (tx_fifo_rden_start or tx_ack_delay or tx_ack);
	n_tx_valid	 <= not tx_valid;

	-- tx_fifo read start signal
	rden_start : process (tx_clk, sys_rst)
	begin
		if sys_rst = '1'  then
			tx_fifo_rden_start <= '0';
		elsif tx_clk'event and tx_clk = '1' then
			tx_fifo_rden_start <= not tx_fifo_empty and (ISOCMBRAMWRDBUS(2) or tx_fifo_full_wire) and n_tx_valid;
		end if;
	end process  rden_start;

	-- last unvalid data read signal
	ack_delay : process (tx_clk)
	begin
		if tx_clk'event and tx_clk = '1' then
			if tx_valid = '0' or sys_rst = '1' then
				tx_ack_delay <= '0';
			elsif tx_ack = '1' then
				tx_ack_delay <= '1';
			end if;
		end if;
	end process ack_delay;

-- rx FIFO supporting logic

	rx_fifo_wren <= (rx_valid or rx_good_frame or rx_bad_frame or (rx_fifo_parity and rx_frame_end_reg))
				and not rx_fifo_full
				and not rx_fifo_reset;

-- connection to  ppc from rx fifo

	BRAMDSOCMRDDBUS(24 to 31)  <= rx_fifo_DO;
	BRAMDSOCMRDDBUS(16 to 23)  <= rx_fifo_DO_2;
	BRAMDSOCMRDDBUS(15) 	   <= '0';
	BRAMDSOCMRDDBUS(14) 	   <= '0';
	BRAMDSOCMRDDBUS(13) 	   <= rx_fifo_full;
	BRAMDSOCMRDDBUS(12) 	   <= '0';
	BRAMDSOCMRDDBUS(11) 	   <= rx_valid_fifo;
	BRAMDSOCMRDDBUS(10) 	   <= rx_valid_fifo_2;
	BRAMDSOCMRDDBUS(9)		   <= '0';
	BRAMDSOCMRDDBUS(8)		   <= rx_fifo_rderr;
	BRAMDSOCMRDDBUS(0 to 7)    <= "00000000";

-----------------------------------------------------------------------------------
--ETHERNET SIGNALS, BUFFERS, FLIPFLIPS, AND EMAC
-----------------------------------------------------------------------------------

	phy_mii_int <= not phy_mii_int_n_IBUF(0);
	ibuf_29 : IBUF port map(
		I => phy_mii_int_n,
		O => phy_mii_int_n_IBUF(0)
	);
 
	obuf55 : OBUF port map(
		I => mdc_0_obuf,
		O => MDC_0
	);
	--*************************** Main Body of Code ****************************
	gnd_i <= '0';

	reset_i <= sys_rst;
	phy_rstn <= ISOCMBRAMWRDBUS(1);

	mdio_reg1 : process (host_clk)
	begin
		if host_clk'event and host_clk = '1' then
		  mdc_0_obuf <= emac_mdc ;
		  MDIO_OUT_0 <=emac_mdio_out;
		  MDIO_TRI_0 <=emac_mdio_tri;
		end if;
	end process mdio_reg1;
	mdio_reg2 : process (mdc_0_obuf)
	begin
		if mdc_0_obuf'event and mdc_0_obuf = '0' then
		  emac_mdio_in <= MDIO_IN_0 ;
		end if;
	end process mdio_reg2;


	--********** EMAC0 **********
	----------------------------------------------------------------------------
	-- MII Transmitter Logic : Drive TX signals through IOBs onto MII
	-- interface
	----------------------------------------------------------------------------
	-- Infer IOB Output flip-flops.
	mii_output_ffs_0 : process (mii_tx_clk_0_i, reset_i)
	begin
		if reset_i = '1' then
			mii_tx_en_0_r <= '0';
			mii_txd_0_r   <= (others => '0');
		elsif mii_tx_clk_0_i'event and mii_tx_clk_0_i = '1' then
			mii_tx_en_0_r <= mii_tx_en_0_i;
			mii_txd_0_r   <= mii_txd_0_i(3 downto 0);
		end if;
	end process mii_output_ffs_0;

	-- Drive MII TX signals through Output Buffers and onto PADS
	mii_tx_en_0_obuf : OBUF port map (I => mii_tx_en_0_r, O => mii_tx_en);
	mii_txd_0_bus: for I in 3 downto 0 generate
		mii_txd_0_obuf : OBUF 
		port map (
			I => mii_txd_0_r(I),
			O => mii_txd(I)
		);
	end generate;

	----------------------------------------------------------------------------
	-- MII Receiver Clock Management
	----------------------------------------------------------------------------
	mii_rx_clk_0_ibufg : IBUFG
	port map (
		I => mii_rx_clk,
		O => mii_rx_clk_ibufg_0_i
	);

	mii_rx_clk_0_bufg : BUFG
	port map (
		I => mii_rx_clk_ibufg_0_i,
		O => mii_rx_clk_0_i
	);

	----------------------------------------------------------------------------
	-- MII Receiver Logic : Receive RX signals through IOBs from MII
	-- interface
	----------------------------------------------------------------------------
	-- Drive input MII Rx signals from PADS through Input Buffers and then 
	-- use IDELAYs to provide Zero-Hold Time Delay ?!?!?
	-- Should the C, CE, INC, and RST ports be connected to GND or left OPEN ?!
	mii_rx_dv_0_ibuf: IBUF port map (I => mii_rx_dv, O => mii_rx_dv_0_i);

	mii_rx_er_0_ibuf: IBUF port map (I => mii_rx_er, O => mii_rx_er_0_i);

	mii_rxd_0_bus: for I in 3 downto 0 generate
		mii_rxd_0_ibuf: IBUF
		port map (
			I => mii_rxd(I),
			O => mii_rxd_0_i(I)
		);
	end generate;

	-- Infer IOB Input flip-flops
	mii_input_ffs_0 : process (mii_rx_clk_0_i, reset_i)
	begin
		if reset_i = '1' then
			mii_rx_dv_0_r <= '0';
			mii_rx_er_0_r <= '0';
			mii_rxd_0_r   <= (others => '0');
		elsif mii_rx_clk_0_i'event and mii_rx_clk_0_i = '1' then
			mii_rx_dv_0_r <= mii_rx_dv_0_i;
			mii_rx_er_0_r <= mii_rx_er_0_i;
			mii_rxd_0_r   <= "0000" & mii_rxd_0_i;
		end if;
	end process mii_input_ffs_0;

	-- MII_TX_CLK
	mii_tx_clk_0_ibufg : BUFG
	port map (
		I => mii_tx_clk,
		O => mii_tx_clk_0_i
	);

	----------------------------------------------------------------------------
	-- Receive & Transmit Client Clock Management when configured in 8-bit mode
	----------------------------------------------------------------------------

	rx_client_clk_0_bufg : BUFG
	port map (
		I => rx_client_clk_out_0_i,
		O => rx_client_clk_in_0_i
	);

	tx_client_clk_0_bufg : BUFG
	port map (
		I => tx_client_clk_out_0_i,
		O => tx_client_clk_in_0_i
	);

	rx_clk <= rx_client_clk_in_0_i;
	tx_clk <= tx_client_clk_in_0_i;

----------------------------------------------------------------------------
-- Instantiate V4 EMAC
----------------------------------------------------------------------------
v4_emac_top : temac
	port map (
		-- Client Receiver Interface - EMAC0
		EMAC0CLIENTRXCLIENTCLKOUT		=> rx_client_clk_out_0_i,
		CLIENTEMAC0RXCLIENTCLKIN		=> rx_client_clk_in_0_i,
		EMAC0CLIENTRXD					=> rx_data,
		EMAC0CLIENTRXDVLD				=> rx_valid,
		EMAC0CLIENTRXDVLDMSW			=> open,
		EMAC0CLIENTRXGOODFRAME			=> rx_good_frame,
		EMAC0CLIENTRXBADFRAME			=> rx_bad_frame,
		EMAC0CLIENTRXFRAMEDROP			=> open,
		EMAC0CLIENTRXDVREG6 			=> open,
		EMAC0CLIENTRXSTATS				=> open,
		EMAC0CLIENTRXSTATSVLD			=> open,
		EMAC0CLIENTRXSTATSBYTEVLD		=> open,

		-- Client Transmitter Interface - EMAC0
		EMAC0CLIENTTXCLIENTCLKOUT		=> tx_client_clk_out_0_i,
		CLIENTEMAC0TXCLIENTCLKIN		=> tx_client_clk_in_0_i,
		CLIENTEMAC0TXD					=> tx_data,
		CLIENTEMAC0TXDVLD				=> tx_valid,
		CLIENTEMAC0TXDVLDMSW			=> '0',
		EMAC0CLIENTTXACK				=> tx_ack,
		CLIENTEMAC0TXFIRSTBYTE			=> '0',
		CLIENTEMAC0TXUNDERRUN			=> '0',
		EMAC0CLIENTTXCOLLISION			=> open,
		EMAC0CLIENTTXRETRANSMIT 		=> open,
		CLIENTEMAC0TXIFGDELAY			=> "00000000",
		EMAC0CLIENTTXSTATS				=> open,
		EMAC0CLIENTTXSTATSVLD			=> open,
		EMAC0CLIENTTXSTATSBYTEVLD		=> open,

		-- MAC Control Interface - EMAC0
		CLIENTEMAC0PAUSEREQ 			=> '0',
		CLIENTEMAC0PAUSEVAL 			=> "0000000000000000",

		-- Clock Signal - EMAC0
		GTX_CLK_0						=> '0',
		EMAC0CLIENTTXGMIIMIICLKOUT		=> open,
		CLIENTEMAC0TXGMIIMIICLKIN		=> mii_tx_clk_0_i,

		-- GMII Interface - EMAC0
		GMII_TXD_0						=> mii_txd_0_i,
		GMII_TX_EN_0					=> mii_tx_en_0_i,
		GMII_TX_ER_0					=> open,
		GMII_TX_CLK_0					=> open,
		GMII_RXD_0						=> mii_rxd_0_r,
		GMII_RX_DV_0					=> mii_rx_dv_0_r,
		GMII_RX_ER_0					=> mii_rx_er_0_r,
		GMII_RX_CLK_0					=> mii_rx_clk_0_i,
		MII_TX_CLK_0					=> mii_tx_clk_0_i,
		GMII_COL_0						=> '0',
		GMII_CRS_0						=> '0',

		-- MDIO Interface - EMAC0
		MDC_0							=> emac_mdc,
		MDIO_IN_0						=> emac_mdio_in,
		MDIO_OUT_0						=> emac_mdio_out,
		MDIO_TRI_0						=> emac_mdio_tri,

		HOSTCLK 						=> host_clk,

		-- Asynchronous Reset
		RESET							=> reset_i
	);

------------------------------------------------------------------
-- rx_fifo : 8k deep x 9 wide BlockRAM Asynchrnous FIFO
--			 9 wide in , 18 wide out
------------------------------------------------------------------
rx_fifo : fifo_9_rx
	port map (
		rst => rx_fifo_reset,
		wr_clk => rx_clk,
		wr_en => rx_fifo_wren,
		din(7 downto 0) => rx_data,
		din(8) => rx_valid,
		full => rx_fifo_full,
		rd_clk => host_clk,
		rd_en => rx_fifo_rden_one,
		dout(7 downto 0) => rx_fifo_DO,
		dout(8) => rx_valid_fifo,
		dout(16 downto 9) => rx_fifo_DO_2,
		dout(17) => rx_valid_fifo_2,
		empty => rx_fifo_empty,
		underflow => rx_fifo_rderr,
		rd_data_count => rx_fifo_rd_data_count
	);

------------------------------------------------------------------
-- tx_fifo : 8k deep x 9 wide BlockRAM Asynchrnous FIFO
--			 18 wide in , 9 wide out
------------------------------------------------------------------
n_tx_fifo_di8	<= not DSOCMBRAMWRDBUS(11);
n_tx_fifo_di8_2 <= not DSOCMBRAMWRDBUS(10);

tx_fifo : fifo_9_tx
	port map (
		rst => sys_rst,
		wr_clk => host_clk,
		wr_en => tx_fifo_wren,
		din(7 downto 0) => tx_fifo_di(7 downto 0),
		din(8) => n_tx_fifo_di8,
		din(16 downto 9) => tx_fifo_di_2(7 downto 0),
		din(17) => n_tx_fifo_di8_2,
		full => tx_fifo_full_wire,
		overflow => tx_fifo_overflow,
		wr_data_count => tx_fifo_wr_data_count,
		rd_clk => tx_clk,
		rd_en => tx_fifo_rden,
		dout(7 downto 0) => tx_data,
		dout(8) => tx_valid,
		empty => tx_fifo_empty
	);

------------------------------------------------------------------
-- rx_done_fifo : 256 deep x 1 wide BlockRAM Asynchrnous FIFO
------------------------------------------------------------------
rx_done_fifo : fifo_1_done
	port map (
		rst => rx_fifo_reset,
		wr_clk => rx_clk,
		wr_en => rx_done_fifo_wren,
		din(0) => '0',
		rd_clk => host_clk,
		rd_en => rx_done_fifo_rden,
		dout(0) => rx_done_fifo_DO,
		empty => rx_done_fifo_empty,
		full => open
	);

end architecture STRUCTURE;
